package com.pseudocode.photolib.metadata;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonProperty;

import com.pseudocode.photolib.utils.GeohashUtil;

/**
 * Simplified (serializable) metadata representation for common image properties.
 */
@JsonIgnoreProperties(ignoreUnknown=true)
public class Meta {

	private String key;
	private String name;
	private String format;
	private String parent;
	private int size;
	private int width;
	private int height;

	// these optional properties are all nullable
	
	private Long timestamp;
	private Integer orientation;
	private Double latitude;
	private Double longitude;
	private Double distance;
	private Double compass;
	private String make;
	private String model;
	private String lens;
	private Double focalLength;
	private Integer iso;
	private Double fstop;
	private String shutter;
	private String exposureBias;
	private Integer exposureMode;
	private String colorSpace;
	private Integer flash;

	private String title;
	private String description;
	private String location;
	private String[] tags;
	private String creator;
	private String copyright;
	private Double rating;

	/**
	 * Gets the unique signature
	 */
	@JsonProperty
	public String key() {
		return key;
	}

	/**
	 * Sets the unique signature
	 */
	@JsonProperty
	public Meta key(String value) {
		key = value;
		return this;
	}

	/**
	 * Gets the original image file name
	 */
	@JsonProperty
	public String name() {
		return name;
	}

	/**
	 * Sets the original image file name
	 */
	@JsonProperty
	public Meta name(String value) {
		name = value;
		return this;
	}

	/**
	 * Gets the original image file format
	 */
	@JsonProperty
	public String format() {
		return format;
	}

	/**
	 * Sets the original image file format
	 */
	@JsonProperty
	public Meta format(String value) {
		format = value;
		return this;
	}

	/**
	 * Gets the image parent directory
	 */
	@JsonProperty
	public String parent() {
		return parent;
	}

	/**
	 * Sets the image parent directory
	 */
	@JsonProperty
	public Meta parent(String value) {
		parent = value;
		return this;
	}

	/**
	 * Gets the original file size in bytes
	 */
	@JsonProperty
	public int size() {
		return size;
	}

	/**
	 * Sets the original file size in bytes
	 */
	@JsonProperty
	public Meta size(int value) {
		size = value;
		return this;
	}

	/**
	 * Gets the image width
	 */
	@JsonProperty
	public int width() {
		return width;
	}

	/**
	 * Sets the image width
	 */
	@JsonProperty
	public Meta width(int value) {
		width = value;
		return this;
	}

	/**
	 * Gets the image height
	 */
	@JsonProperty
	public int height() {
		return height;
	}

	/**
	 * Sets the image height
	 */
	@JsonProperty
	public Meta height(int value) {
		height = value;
		return this;
	}

	/**
	 * Gets the image time stamp
	 */
	@JsonProperty
	public Long timestamp() {
		return timestamp;
	}

	/**
	 * Sets the image time stamp
	 */
	@JsonProperty
	public Meta timestamp(Long value) {
		timestamp = value;
		return this;
	}

	/**
	 * Gets the camera orientation
	 */
	@JsonProperty
	public Integer orientation() {
		return orientation;
	}

	/**
	 * Sets the camera orientation
	 */
	@JsonProperty
	public Meta orientation(Integer value) {
		orientation = value;
		return this;
	}

	/**
	 * Gets the geohash
	 */
	@JsonProperty
	public String geohash() {
		if ((latitude == null) || (longitude == null)) {
			return null;
		}
		return GeohashUtil.encodeHash(latitude, longitude);
	}

	/**
	 * Sets the geohash
	 */
	@JsonProperty
	public Meta geohash(String value) {
		// ignored as is a derived property
		return this;
	}

	/**
	 * Gets the latitude
	 */
	@JsonProperty
	public Double latitude() {
		return latitude;
	}

	/**
	 * Sets the latitude
	 */
	@JsonProperty
	public Meta latitude(Double value) {
		latitude = value;
		return this;
	}

	/**
	 * Gets the longitude
	 */
	@JsonProperty
	public Double longitude() {
		return longitude;
	}

	/**
	 * Sets the longitude
	 */
	@JsonProperty
	public Meta longitude(Double value) {
		longitude = value;
		return this;
	}

	/**
	 * Gets the distance from center (lat/lon) in KM
	 */
	@JsonProperty
	public Double distance() {
		return distance;
	}

	/**
	 * Sets the distance from center (lat/lon) in KM
	 */
	@JsonProperty
	public Meta distance(Double value) {
		distance = value;
		return this;
	}

	/**
	 * Gets the compass heading
	 */
	@JsonProperty
	public Double compass() {
		return compass;
	}

	/**
	 * Sets the compass heading
	 */
	@JsonProperty
	public Meta compass(Double value) {
		compass = value;
		return this;
	}

	/**
	 * Gets the camera make
	 */
	@JsonProperty
	public String make() {
		return make;
	}

	/**
	 * Sets the camera make
	 */
	@JsonProperty
	public Meta make(String value) {
		make = value;
		return this;
	}

	/**
	 * Gets the camera model
	 */
	@JsonProperty
	public String model() {
		return model;
	}

	/**
	 * Sets the camera model
	 */
	@JsonProperty
	public Meta model(String value) {
		model = value;
		return this;
	}

	/**
	 * Gets the camera lens
	 */
	@JsonProperty
	public String lens() {
		return lens;
	}

	/**
	 * Sets the camera lens
	 */
	@JsonProperty
	public Meta lens(String value) {
		lens = value;
		return this;
	}

	/**
	 * Gets the lens focal length
	 */
	@JsonProperty
	public Double focalLength() {
		return focalLength;
	}

	/**
	 * Sets the lens focal length
	 */
	@JsonProperty
	public Meta focalLength(Double value) {
		focalLength = value;
		return this;
	}

	/**
	 * Gets the ISO settings
	 */
	@JsonProperty
	public Integer iso() {
		return iso;
	}

	/**
	 * Sets the ISO settings
	 */
	@JsonProperty
	public Meta iso(Integer value) {
		iso = value;
		return this;
	}

	/**
	 * Gets the aperture settings
	 */
	@JsonProperty
	public Double fstop() {
		return fstop;
	}

	/**
	 * Sets the aperture settings
	 */
	@JsonProperty
	public Meta fstop(Double value) {
		fstop = value;
		return this;
	}

	/**
	 * Gets the shutter speed
	 */
	@JsonProperty
	public String shutter() {
		return shutter;
	}

	/**
	 * Sets the shutter speed
	 */
	@JsonProperty
	public Meta shutter(String value) {
		shutter = value;
		return this;
	}

	/**
	 * Gets the exposure bias settings
	 */
	@JsonProperty
	public String exposureBias() {
		return exposureBias;
	}

	/**
	 * Sets the exposure bias settings
	 */
	@JsonProperty
	public Meta exposureBias(String value) {
		exposureBias = value;
		return this;
	}

	/**
	 * Gets the exposure mode settings
	 */
	@JsonProperty
	public Integer exposureMode() {
		return exposureMode;
	}

	/**
	 * Sets the exposure mode settings
	 */
	@JsonProperty
	public Meta exposureMode(Integer value) {
		exposureMode = value;
		return this;
	}

	/**
	 * Gets the color space settings
	 */
	@JsonProperty
	public String colorSpace() {
		return colorSpace;
	}

	/**
	 * Sets the color space settings
	 */
	@JsonProperty
	public Meta colorSpace(String value) {
		colorSpace = value;
		return this;
	}

	/**
	 * Gets the flash settings
	 */
	@JsonProperty
	public Integer flash() {
		return flash;
	}

	/**
	 * Sets the flash settings
	 */
	@JsonProperty
	public Meta flash(Integer value) {
		flash = value;
		return this;
	}

	/**
	 * Gets the title
	 */
	@JsonProperty
	public String title() {
		return title;
	}

	/**
	 * Sets the title
	 */
	@JsonProperty
	public Meta title(String value) {
		title = value;
		return this;
	}

	/**
	 * Gets the description
	 */
	@JsonProperty
	public String description() {
		return description;
	}

	/**
	 * Sets the description
	 */
	@JsonProperty
	public Meta description(String value) {
		description = value;
		return this;
	}

	/**
	 * Gets the location
	 */
	@JsonProperty
	public String location() {
		return location;
	}

	/**
	 * Sets the location
	 */
	@JsonProperty
	public Meta location(String value) {
		location = value;
		return this;
	}

	/**
	 * Gets the list of tags
	 */
	@JsonProperty
	public String[] tags() {
		return tags;
	}

	/**
	 * Sets the list of tags
	 */
	@JsonProperty
	public Meta tags(String[] value) {
		tags = value;
		return this;
	}

	/**
	 * Gets the creator
	 */
	@JsonProperty
	public String creator() {
		return creator;
	}

	/**
	 * Sets the creator
	 */
	@JsonProperty
	public Meta creator(String value) {
		creator = value;
		return this;
	}

	/**
	 * Gets the copyright
	 */
	@JsonProperty
	public String copyright() {
		return copyright;
	}

	/**
	 * Sets the copyright
	 */
	@JsonProperty
	public Meta copyright(String value) {
		copyright = value;
		return this;
	}

	/**
	 * Gets the photo rating
	 */
	@JsonProperty
	public Double rating() {
		return rating;
	}

	/**
	 * Sets the photo rating
	 */
	@JsonProperty
	public Meta rating(Double value) {
		rating = value;
		return this;
	}
}
